#ifndef BENVUTIL_H_INCLUDED
#define BENVUTIL_H_INCLUDED

/* Common arg processing for message and other sizes (getsizes.c) */
int *BENV_RstringToArrayOLD(char *rstring, int *nsizes);
int *BENV_RstringToArray(const char *rstring, int *nsizes);
int *BENV_GetSizes(int argc, char *argv[], const char *argname, int *nsizes);
int *BENV_GetSizesArith(int start, int end, int stride, int *nsizes);
int *BENV_GetSizesMult(int start, int end, double factor, int *nsizes);
int *BENV_GetSizesMultDelta(int start, int end, double factor, int delta,
			    int ndelta, int *nsizes);

int BENV_GetNrepsEst(double deltat, double latency, double bandwidth,
		     int nmsgs, int msglen, int minreps);

#ifdef BENV_WITHMPI
/* Communication strings in MPI (mstring.c) */
const char *BENV_ConcatStringsFree(const char *s1, const char *s2);
int BENV_SendString(const char *s, int dest, int tag, MPI_Comm comm);
const char *BENV_RecvString(int source, int tag, MPI_Comm comm);

/* HW description (hwdescutil.c) */
/* This is a distributed data structure for each process that identifies
   the process in a hierarchy of communicators, and provides information
   for communicating with the sibling communicators at each level
*/
typedef struct hwdesc_t {
    MPI_Comm comm;          /* Communicator for all processes in the same
			       grouping at this level */
    const char *descstr;    /* Name of this level */
    int      nDistinct;     /* Number of distinct communicators at this level */
    int      idx;           /* Index of this communicator relative to leaders.
			       That is, this process belongs to the same comm
			       whose leader process (rank 0 in comm) has
			       rank leaders[idx] in the parent communicator */
    int      *leaders;      /* ranks in the parent of the leaders (processes
			       with rank 0) of the communicators at this
			       level */
    int      allsizeone;    /* True if all communicators at this level are
			       size 1; common for "core" hw type */
} hwdesc_t;

int BENV_HwdescGetLocal(MPI_Comm comm, hwdesc_t hw[], int maxdepth, int *depth);
int BENV_HwdescPrintArray(FILE *fp, hwdesc_t *hw, int hwdepth,
			  const char *prefix);
int BENV_HwdescPrintAll(FILE *fp, MPI_Comm pcomm, hwdesc_t *hw, int hwdepth,
			int depth);
int BENVi_FindLeadersInSplit(MPI_Comm pcomm, MPI_Comm comm, hwdesc_t *hw);
/* util/nodeinfo.c */
int BENV_NodeGetComms(MPI_Comm comm, hwdesc_t hw[], int maxdepth, int *depth);
int BENV_HwdescGetByShared(MPI_Comm comm, hwdesc_t hw[], int maxdepth,
			       int *depth);
int BENV_NodeCvarInit(void);
void BENV_NodeCvarSet(const char *name, int value);
#endif
#endif /* BENVUTIL_H */

